/*
 * Copyright notice from original mutt:
 * Copyright (C) 2004 g10 Code GmbH
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#ifndef CRYPT_GPGME_H
#define CRYPT_GPGME_H

#include <lib-crypt/crypt.h>

/* {{{ cruft */

/* Type defintions for crypto module functions. */
typedef int (*crypt_func_decrypt_mime_t) (FILE * a, FILE ** b,
                                          BODY * c, BODY ** d);

typedef int (*crypt_func_application_handler_t) (BODY * m, STATE * s);
typedef int (*crypt_func_encrypted_handler_t) (BODY * m, STATE * s);

typedef void (*crypt_func_pgp_invoke_getkeys_t) (address_t * addr);
typedef int (*crypt_func_pgp_check_traditional_t) (FILE * fp, BODY * b,
                                                   int tagged_only);
typedef BODY *(*crypt_func_pgp_traditional_encryptsign_t) (BODY * a,
                                                           int flags,
                                                           char *keylist);
typedef BODY *(*crypt_func_pgp_make_key_attachment_t) (char *tempf);
typedef char *(*crypt_func_findkeys_t) (address_t * to,
                                        address_t * cc, address_t * bcc);
typedef BODY *(*crypt_func_sign_message_t) (BODY * a);
typedef BODY *(*crypt_func_pgp_encrypt_message_t) (BODY * a, char *keylist,
                                                   int sign);
typedef void (*crypt_func_pgp_invoke_import_t) (const char *fname);
typedef int (*crypt_func_verify_one_t) (BODY * sigbdy, STATE * s,
                                        const char *tempf);
typedef void (*crypt_func_pgp_extract_keys_from_attachment_list_t)
     (FILE * fp, int tag, BODY * top);

typedef int (*crypt_func_send_menu_t) (HEADER * msg, int *redraw);

 /* (SMIME) */
typedef void (*crypt_func_smime_getkeys_t) (ENVELOPE * env);
typedef int (*crypt_func_smime_verify_sender_t) (HEADER * h);

typedef BODY *(*crypt_func_smime_build_smime_entity_t) (BODY * a,
                                                        char *certlist);

typedef void (*crypt_func_smime_invoke_import_t) (char *infile,
                                                  char *mailbox);

typedef void (*crypt_func_init_t) (void);


typedef struct crypt_module_specs {
    /* Common/General functions.  */
    crypt_func_init_t init;
    crypt_func_decrypt_mime_t decrypt_mime;
    crypt_func_application_handler_t application_handler;
    crypt_func_encrypted_handler_t encrypted_handler;
    crypt_func_findkeys_t findkeys;
    crypt_func_sign_message_t sign_message;
    crypt_func_verify_one_t verify_one;
    crypt_func_send_menu_t send_menu;

    /* PGP specific functions.  */
    crypt_func_pgp_encrypt_message_t pgp_encrypt_message;
    crypt_func_pgp_make_key_attachment_t pgp_make_key_attachment;
    crypt_func_pgp_check_traditional_t pgp_check_traditional;
    crypt_func_pgp_traditional_encryptsign_t pgp_traditional_encryptsign;
    crypt_func_pgp_invoke_getkeys_t pgp_invoke_getkeys;
    crypt_func_pgp_invoke_import_t pgp_invoke_import;

    crypt_func_pgp_extract_keys_from_attachment_list_t
        pgp_extract_keys_from_attachment_list;

    /* S/MIME specific functions.  */

    crypt_func_smime_getkeys_t smime_getkeys;
    crypt_func_smime_verify_sender_t smime_verify_sender;
    crypt_func_smime_build_smime_entity_t smime_build_smime_entity;
    crypt_func_smime_invoke_import_t smime_invoke_import;
} crypt_module_specs_t;

extern struct crypt_module_specs crypt_mod_SMIME_gpgme;
extern struct crypt_module_specs crypt_mod_PGP_gpgme;

#define CRYPT_MOD_CALL_CHECK(identifier, func) \
    ((crypt_mod_##identifier##_gpgme).func)

#define CRYPT_MOD_CALL(identifier, func) \
    (*((crypt_mod_##identifier##_gpgme).func))


/* }}} */

void pgp_gpgme_init (void);
void smime_gpgme_init (void);

char *pgp_gpgme_findkeys (address_t * to, address_t * cc, address_t * bcc);
char *smime_gpgme_findkeys (address_t * to, address_t * cc, address_t * bcc);

BODY *pgp_gpgme_encrypt_message (BODY * a, char *keylist, int sign);
BODY *smime_gpgme_build_smime_entity (BODY * a, char *keylist);

int pgp_gpgme_decrypt_mime (FILE * fpin, FILE ** fpout, BODY * b,
                            BODY ** cur);
int smime_gpgme_decrypt_mime (FILE * fpin, FILE ** fpout, BODY * b,
                              BODY ** cur);

int pgp_gpgme_check_traditional (FILE * fp, BODY * b, int tagged_only);

int pgp_gpgme_application_handler (BODY * m, STATE * s);
int smime_gpgme_application_handler (BODY * a, STATE * s);
int pgp_gpgme_encrypted_handler (BODY * a, STATE * s);

BODY *pgp_gpgme_make_key_attachment (char *tempf);

BODY *pgp_gpgme_sign_message (BODY * a);
BODY *smime_gpgme_sign_message (BODY * a);

int pgp_gpgme_verify_one (BODY * sigbdy, STATE * s, const char *tempfile);
int smime_gpgme_verify_one (BODY * sigbdy, STATE * s, const char *tempfile);

int smime_gpgme_verify_sender (HEADER* h);

int pgp_gpgme_send_menu (HEADER * msg, int *redraw);
int smime_gpgme_send_menu (HEADER * msg, int *redraw);
void pgp_gpgme_invoke_import(const char *fname);
void pgp_gpgme_from_attachment_list (FILE * fp, int tag, BODY * top);

#endif
